#ifndef I2C_H
#define I2C_H

#include "i2c-gpio.hpp"
// https://www.st.com/resource/en/reference_manual/dm00031936-stm32f0x1stm32f0x2stm32f0x8-advanced-armbased-32bit-mcus-stmicroelectronics.pdf

// The default address for an encoder that's just been turrned on.
#define I2CENCODER_DEFAULT_ADDRESS 0x60

// Registers for important data
// See : https://www.vexforum.com/t/vex-i2c-integrated-motor-encoder-device-spec/20600

// Write to this register to change the addresse of the vex encoder
// By default the adress is 0x60
#define I2CENCODER_ADDRESS_REGISTER 0x4D

// Read these registers to get data from the encoder
#define I2CENCODER_POSITION_REGISTER 0x40
#define I2CENCODER_VELOCITY_REGISTER 0x44

// Write to this register to zero the encoder
#define I2CENCODER_ZERO_REGISTER 0x4A

// Using address 0x00 write to this register by sending bytes 0xCA and 0x03
#define I2CENCODER_RESET_ALL_DEVICES 0x4E

// Write to these registers to Terminate/Unterminate encoder. Only usefull when daisy chaining encoders
#define I2CENCODER_UNTERMINATE_REGISTER 0x4B
#define I2CENCODER_TERMINATE_REGISTER 0x4C

namespace stm32f072xb {

class I2c {
   private:
	I2C_TypeDef* _type;

   public:
	I2c(I2C_TypeDef* type);
	void blockingWrite(uint8_t slaveAddress, uint8_t* bytes, uint8_t bufferSize);
	void blockingRead(uint8_t slaveAddress, uint8_t* bytes, uint8_t bufferSize);
	void write(uint8_t slaveAddress, uint8_t bufferSize);
	void read(uint8_t slaveAddress, uint8_t bufferSize);
	void softwareReset();

	void enableDmaReception();
	void enableDmaTransmission();

   private:
	void setupI2c();
	void configureI2cClock();
	void setupGpio();
	void enableI2cClock();
	void setupTransmission(uint8_t slaveAddress, uint8_t nBytes);
};

}  // namespace stm32f072xb

#endif
