#ifndef STEPPER_H
#define STEPPER_H

#include <alternate-gpio.hpp>
#include <output-gpio.hpp>

#include "microstep-resolution.hpp"
#include "motor-direction.hpp"
#include "pwm-timer.hpp"
#include "stepper-init.hpp"

using namespace stm32f072xb;

template <Port pwmPort, Port directionPort, Port disablePort>
class Stepper {
   public:
	static const uint16_t PWM_RESOLUTION = 1000;
	static const uint16_t PWM_FREQUENCY = 1000;

   public:
	Stepper(StepperInit stepperInit)
	    : _timer(stepperInit.timer, stepperInit.OCC, PWM_RESOLUTION, PWM_FREQUENCY),
	      _pwmOutputPin({stepperInit.pwmPin, Pupdr::NONE, stepperInit.alternateFunction}),
	      _directionPin({stepperInit.directionPin, Pupdr::NONE}),
	      _disablePort({stepperInit.disablePort, Pupdr::NONE}) {
		_timer.setDutyCyclePercentage(50);
	}

	void setDirection(MotorDirection direction) {
		if (direction == MotorDirection::CLOCK_WISE) {
			_directionPin.setPin(FlagStatus::SET);
		} else {
			_directionPin.setPin(FlagStatus::RESET);
		}
	}

	void start() { _disablePort.setPin(FlagStatus::RESET); }

	void stop() { _disablePort.setPin(FlagStatus::SET); }

   private:
	PwmTimer _timer;
	AlternateGpio<pwmPort> _pwmOutputPin;
	OutputGpio<directionPort> _directionPin;
	OutputGpio<disablePort> _disablePort;
};

#endif
